using System;
using System.Collections;
using System.Drawing;

public enum Modes { Create, Edit, Delete };
public delegate void StateChangedHandler();

class BezierModel
{
    // Members
    private ArrayList controlPoints;
	private ArrayList curvePoints;
	private ArrayList constructionPoints;
    private Modes mode;
    public event StateChangedHandler StateChanged;
	private double t;
	private int resolution;
	private Point bezierPoint;
    
    // C'tor
    public BezierModel()
    {
        this.mode = Modes.Create;
        this.controlPoints = new ArrayList();
		this.curvePoints = new ArrayList();
		this.constructionPoints = new ArrayList();
		this.t = 0.5;
		this.resolution = 50;
    }
    
    // Getter / Setter
    public ArrayList ControlPoints
    {
        get {return new ArrayList(this.controlPoints);}
    }

	public ArrayList CurvePoints
	{
		get {return new ArrayList(this.curvePoints);}
	}

	public ArrayList ConstructionPoints
	{
		get {return new ArrayList(this.constructionPoints);}
	}
    
    public Modes Mode
    {
		get {return this.mode;}
    }

	public Point BezierPoint
	{
		get {return this.bezierPoint;}
	}
    
    // Methods
    public void SetMode(Modes m)
    {
		this.mode = m;

		if (this.StateChanged != null)
			this.StateChanged();
    }

	public void SetT(double t)
	{
		this.t = t;
		
		this.DeCasteljau();
		this.getCurve();

		if (this.StateChanged != null)
			this.StateChanged();
	}

    public void SetResolution(int res)
    {
        this.resolution = res;

        this.DeCasteljau();
        this.getCurve();

        if (this.StateChanged != null)
            this.StateChanged();
    }
    
    public void Clear()
    {
        this.controlPoints.Clear();
		this.curvePoints.Clear();
		this.constructionPoints.Clear();		
		this.bezierPoint.X = this.bezierPoint.Y = 0;
        
        if (this.StateChanged != null)
            this.StateChanged();
    }
    
    public void AddPoint(Point p)
    {
        this.controlPoints.Add(p);

		this.DeCasteljau();
        this.getCurve();
        
        if (this.StateChanged != null)
            this.StateChanged();
    }
    
    public void RemovePoint(Point p)
    {
        this.controlPoints.Remove(p);

		this.DeCasteljau();
        this.getCurve();

        if (this.StateChanged != null)
            this.StateChanged();
    }
    
    public void MovePoint(int i, Point p)
    {
		this.controlPoints[i] = p;

		this.DeCasteljau();
        this.getCurve();

		if (this.StateChanged != null)
			this.StateChanged();
    }

	private void DeCasteljau()
	{
		if (this.controlPoints.Count == 0)
			return;

		Point[] Q = (Point[]) this.controlPoints.ToArray(typeof(Point));
		this.constructionPoints.Clear();

		while(Q.Length > 1)
		{
			ArrayList a = new ArrayList();

			for (int i=0; i<Q.Length-1; i++)
			{
				Point p = new Point();

				p.X = Q[i].X + (int)(((double)(Q[i+1].X - Q[i].X)) * this.t);
				p.Y = Q[i].Y + (int)(((double)(Q[i+1].Y - Q[i].Y)) * this.t);

				a.Add(p);
			}

			this.constructionPoints.Add(a);
			Q = (Point[]) a.ToArray(typeof(Point));
		}

		this.bezierPoint.X = Q[0].X;
		this.bezierPoint.Y = Q[0].Y;
	}

    private void getCurve()
    {
        if (this.controlPoints.Count == 0)
            return;

        Point[] Q = (Point[]) this.controlPoints.ToArray(typeof(Point));
        this.curvePoints.Clear();
        this.curvePoints.Add(this.controlPoints[0]);
        
        double start = ((double)1) / this.resolution;
        
        for (double i=start; i<(double)1; i+=start)
        {
            while (Q.Length > 1)
            {
                ArrayList a = new ArrayList();

                for (int ii = 0; ii < Q.Length - 1; ii++)
                {
                    Point p = new Point();

                    p.X = Q[ii].X + (int)(((double)(Q[ii + 1].X - Q[ii].X)) * i);
                    p.Y = Q[ii].Y + (int)(((double)(Q[ii + 1].Y - Q[ii].Y)) * i);

                    a.Add(p);
                }

                Q = (Point[])a.ToArray(typeof(Point));
            }
            
            this.curvePoints.Add(Q[0]);
            Q = (Point[])this.controlPoints.ToArray(typeof(Point));
        }

        this.curvePoints.Add(this.controlPoints[this.controlPoints.Count-1]);
    }
}